@@ -1,7 +1,7 @@
 #
 # common.py: common classes for ufw
 #
-# Copyright 2008-2009 Canonical Ltd.
+# Copyright 2008-2010 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -23,7 +23,8 @@ from ufw.util import debug
 
 programName = "ufw"
 state_dir = "#STATE_PREFIX#"
-trans_dir = "#TRANSLATIONS_PREFIX#"
+share_dir = "#SHARE_DIR#"
+trans_dir = share_dir
 config_dir = "#CONFIG_PREFIX#"
 prefix_dir = "#PREFIX#"
 iptables_dir = "#IPTABLES_DIR#"
@@ -72,6 +73,13 @@ class UFWRule:
     def __str__(self):
         return self.format_rule()
 
+    def _get_attrib(self):
+        '''Print rule to stdout'''
+        res = "'%s'" % (self)
+        for k in self.__dict__:
+            res += ", %s=%s" % (k, self.__dict__[k])
+        return res
+
     def dup_rule(self):
         '''Return a duplicate of a rule'''
         rule = UFWRule(self.action, self.protocol)
@@ -232,7 +240,12 @@ class UFWRule:
 
     def set_protocol(self, protocol):
         '''Sets protocol of the rule'''
-        if protocol == "tcp" or protocol == "udp" or protocol == "any":
+        if protocol == "tcp" or \
+           protocol == "udp" or \
+           protocol == "ipv6" or \
+           protocol == "esp" or \
+           protocol == "ah" or \
+           protocol == "any":
             self.protocol = protocol
         else:
             err_msg = _("Unsupported protocol '%s'") % (protocol)
@@ -284,7 +297,7 @@ class UFWRule:
             err_msg = _("Bad interface type")
             raise UFWError(err_msg)
 
-        if not re.match(r'^[a-zA-Z][a-zA-Z0-9:]*[0-9]', str(name)):
+        if not re.match(r'^[a-zA-Z][a-zA-Z0-9:]*[a-zA-Z0-9]', str(name)):
             err_msg = _("Bad interface name")
             raise UFWError(err_msg)
 
@@ -366,7 +379,7 @@ class UFWRule:
         if not x or not y:
             raise ValueError()
 
-        dbg_msg = _("No match")
+        dbg_msg = "No match '%s' '%s'" % (x, y)
         if x.dport != y.dport:
             debug(dbg_msg)
             return 1
@@ -392,15 +405,19 @@ class UFWRule:
             debug(dbg_msg)
             return 1
         if x.interface_in != y.interface_in:
+            debug(dbg_msg)
             return 1
         if x.interface_out != y.interface_out:
+            debug(dbg_msg)
             return 1
         if x.direction != y.direction:
+            debug(dbg_msg)
             return 1
         if x.action == y.action and x.logtype == y.logtype:
             dbg_msg = _("Found exact match")
             debug(dbg_msg)
             return 0
+
         dbg_msg = _("Found non-action/non-logtype match " \
                     "(%(xa)s/%(ya)s %(xl)s/%(yl)s)") % \
                     ({'xa': x.action, 'ya': y.action, \
@@ -408,6 +425,106 @@ class UFWRule:
         debug(dbg_msg)
         return -1
 
+    def fuzzy_dst_match(x, y):
+	'''This will match if x is more specific than y. Eg, for protocol if x
+	   is tcp and y is all or for address if y is a network and x is a
+           subset of y (where x is either an address or network). Returns:
+
+            0  match
+            1  no match
+           -1  fuzzy match
+
+           This is a fuzzy destination match, so source ports or addresses
+           are not considered, and (currently) only incoming.
+        '''
+        def _match_ports(p, to_match):
+            '''Returns True if p is an exact match or within a multi rule'''
+            for port in to_match.split(','):
+                if p == port:
+                    return True
+                if ':' in port:
+                    (low, high) = port.split(':')
+                    if int(p) >= int(low) and int(p) <= int(high):
+                        return True
+
+            return False
+
+        if not x or not y:
+            raise ValueError()
+
+        # Ok if exact match
+        if x.match(y) == 0:
+            return 0
+
+        dbg_msg = "No fuzzy match '%s (v6=%s)' '%s (v6=%s)'" % \
+                   (x, x.v6, y, y.v6)
+
+        # Direction must match
+        if y.direction != "in":
+            debug("(direction) " + dbg_msg + " (not incoming)")
+            return 1
+
+        # Protocols must match or y 'any'
+        if x.protocol != y.protocol and y.protocol != "any":
+            debug("(protocol) " + dbg_msg)
+            return 1
+
+        # Destination ports must match or y 'any'
+        if y.dport != "any" and not _match_ports(x.dport, y.dport):
+            debug("(dport) " + dbg_msg)
+            return 1
+
+        if y.interface_in == "":
+	    # If destination interface is not specified, destination addresses
+            # must match or x must be contained in y
+
+            if x.interface_in == "" and x._is_anywhere(x.dst):
+		# if x and y interfaces are not specified, and x.dst is
+                # anywhere then ok
+                pass
+            elif x.dst != y.dst and '/' not in y.dst:
+                debug("(dst) " + dbg_msg)
+                return 1
+            elif x.dst != y.dst and '/' in y.dst and x.v6 == y.v6 and \
+               not ufw.util.in_network(x.dst, y.dst, x.v6):
+                debug("(dst) " + dbg_msg + " ('%s' not in network '%s')" % \
+                      (x.dst, y.dst))
+                return 1
+        else:
+	    # If destination interface is specified, then:
+            #  if specified, both interfaces must match or
+            #  the IP of the interface must match the IP of y or
+	    #  the IP of the interface must be contained in y
+            if x.interface_in != "" and x.interface_in != y.interface_in:
+                debug("(interface) " + dbg_msg + " (%s != %s)" % \
+                      (x.interface_in, y.interface_in))
+                return 1
+
+            if_ip = ufw.util.get_ip_from_if(y.interface_in, x.v6)
+            if y.dst != if_ip and '/' not in y.dst:
+                debug("(interface) " + dbg_msg + " (%s != %s)" % \
+                      (y.dst, if_ip))
+                return 1
+            elif y.dst != if_ip and '/' in y.dst and x.v6 == y.v6 and \
+               not ufw.util.in_network(if_ip, y.dst, x.v6):
+                debug("(interface) " + dbg_msg + \
+                      " ('%s' not in network '%s')" % (if_ip, y.dst))
+                return 1
+
+        if x.v6 != y.v6:
+            debug("(v6) " + dbg_msg + " (%s != %s)" % (x.dst, y.dst))
+            return 1
+
+        # if we made it here, it is a fuzzy match
+        debug("(fuzzy match) '%s (v6=%s)' '%s (v6=%s)'" % (x, x.v6, y, y.v6))
+        return -1
+
+    def _is_anywhere(self, addr):
+        '''Check if address is anywhere'''
+        if addr == "::/0" or addr == "0.0.0.0/0":
+            return True
+        return False
+
     def get_app_tuple(self):
         '''Returns a tuple to identify an app rule. Tuple is:
              dapp dst sapp src
