@@ -143,7 +143,7 @@ do
         done
 done
 
-do_cmd "0" null --dry-run allow to 2001:0db8:0000:0000:0000:0000:0000:0001/128
+do_cmd "0" null --dry-run allow to 2001:db8:0000:0000:0000:0000:0000:0001/128
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
 
 
@@ -296,17 +296,17 @@ do_cmd "0" delete allow from ::1/32 to ::1/128
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
 
 echo "TESTING MULTIPORT" >> $TESTTMP/result
-do_cmd "0" allow to 2001:0db8:85a3:08d3:1319:8a2e:0370:7341 port 80:83 proto tcp
+do_cmd "0" allow to 2001:db8:85a3:8d3:1319:8a2e:370:7341 port 80:83 proto tcp
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
-do_cmd "0" delete allow to 2001:0db8:85a3:08d3:1319:8a2e:0370:7341 port 80:83 proto tcp
+do_cmd "0" delete allow to 2001:db8:85a3:8d3:1319:8a2e:370:7341 port 80:83 proto tcp
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
-do_cmd "0" allow to 2001:0db8:85a3:08d3:1319:8a2e:0370:7341 port 80:83,22 proto tcp
+do_cmd "0" allow to 2001:db8:85a3:8d3:1319:8a2e:370:7341 port 80:83,22 proto tcp
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
-do_cmd "0" delete allow to 2001:0db8:85a3:08d3:1319:8a2e:0370:7341 port 80:83,22 proto tcp
+do_cmd "0" delete allow to 2001:db8:85a3:8d3:1319:8a2e:370:7341 port 80:83,22 proto tcp
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
-do_cmd "0" allow from 2001:0db8:85a3:08d3:1319:8a2e:0370:7341 port 35:39 to 2001:0db8:85a3:08d3:1319:8a2e:0370:7342 port 22 proto tcp
+do_cmd "0" allow from 2001:db8:85a3:8d3:1319:8a2e:370:7341 port 35:39 to 2001:db8:85a3:8d3:1319:8a2e:370:7342 port 22 proto tcp
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
-do_cmd "0" delete allow from 2001:0db8:85a3:08d3:1319:8a2e:0370:7341 port 35:39 to 2001:0db8:85a3:08d3:1319:8a2e:0370:7342 port 22 proto tcp
+do_cmd "0" delete allow from 2001:db8:85a3:8d3:1319:8a2e:370:7341 port 35:39 to 2001:db8:85a3:8d3:1319:8a2e:370:7342 port 22 proto tcp
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
 do_cmd "0" allow to any port 23,21,15:19,22 from any port 24:26 proto udp
 grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
@@ -321,5 +321,53 @@ do_cmd "0" delete allow 23,21,15:19,22/udp
 grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
 grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
 
+echo "TESTING IPSec" >> $TESTTMP/result
+do_cmd "0" allow to 10.0.0.1 proto esp
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" delete allow to 10.0.0.1 proto esp
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" deny to 10.0.0.1 from 10.4.0.0/16 proto esp
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" delete deny to 10.0.0.1 from 10.4.0.0/16 proto esp
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" limit to 10.0.0.1 proto ah
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" delete limit to 10.0.0.1 proto ah
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" reject to 10.0.0.1 from 10.4.0.0/16 proto ah
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+do_cmd "0" delete reject to 10.0.0.1 from 10.4.0.0/16 proto ah
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+
+do_cmd "0" reject to 2001:db8:85a3:8d3:1319:8a2e:370:734 proto esp
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" delete reject to 2001:db8:85a3:8d3:1319:8a2e:370:734 proto esp
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" allow to 2001:db8:85a3:8d3:1319:8a2e:370:734 from 2001:db8::/32 proto esp
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" delete allow to 2001:db8:85a3:8d3:1319:8a2e:370:734 from 2001:db8::/32 proto esp
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" deny to 2001:db8:85a3:8d3:1319:8a2e:370:734 proto ah
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" delete deny to 2001:db8:85a3:8d3:1319:8a2e:370:734 proto ah
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" allow to 2001:db8:85a3:8d3:1319:8a2e:370:734 from 2001:db8::/32 proto ah
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" delete allow to 2001:db8:85a3:8d3:1319:8a2e:370:734 from 2001:db8::/32 proto ah
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+
+do_cmd "0" allow to any proto esp
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" delete allow to any proto esp
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" deny to any proto ah
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+do_cmd "0" delete deny to any proto ah
+grep -A2 "tuple" $TESTSTATE/user.rules >> $TESTTMP/result
+grep -A2 "tuple" $TESTSTATE/user6.rules >> $TESTTMP/result
+
 cleanup
 exit 0
